@@ -99,7 +99,7 @@ sub canonpath (Str $_path) returns Str is export {
     # trait at some point
     my $path = $_path;
     my $orig_path = $path;
-    $path ~~ s:perl5{^([a-z]:)}{{uc$0}};
+    $path ~~ s:perl5[^([a-z]:)][{uc$0}];
     $path ~~ s:perl5:g{/}{\\};
     $path ~~ s:perl5:g{([^\\])\\+}{$0\\};                                                 # xx\\\\xx  -> xx\xx
     $path ~~ s:perl5:g{(\\\.)+\\}{\\};                                           # xx\.\.\xx -> xx\xx
@@ -309,4 +309,4 @@ it under the same terms as Perl itself.
 
 See http://www.perl.com/perl/misc/Artistic.html
 
-=cut
\ No newline at end of file
+=cut
