@@ -28,7 +28,7 @@ evalEmbedded "Parrot"  code = do
     evalHaskell code
     return () -}
 evalEmbedded "Perl5" code = do
-    interp <- initPerl5 ""
-    evalPerl5 code
+    interp <- initPerl5 "" (Nothing :: Maybe ())
+    evalPerl5 code nullSV 0
     freePerl5 interp
 evalEmbedded s _ = fail $ "Cannot evaluate in " ++ s
