@@ -155,7 +155,7 @@ joinDashE ((Switch 'n'):args) = joinDashE ((Opt "-e" "while ($_ = =<>) { chomp $
                                    isDashE (Opt "-e" _) = True
                                    isDashE (_) = False
 
-joinDashE ((Opt "-M" mod):args) = joinDashE ((Opt "-e" ("require " ++ mod ++ ";\n")):args)
+joinDashE ((Opt "-M" mod):args) = joinDashE ((Opt "-e" ("use " ++ mod ++ ";\n")):args)
 
 joinDashE ((Opt "-e" a):(Opt "-e" b):args) =
     joinDashE (Opt "-e" combined:args)
