@@ -21,6 +21,7 @@ data Command
    = CmdLoad FilePath
    | CmdQuit
    | CmdParse String
+   | CmdParseRaw String
    | CmdRun RunOptions String
    | CmdHelp
    | CmdReset
@@ -29,13 +30,14 @@ data RunOptions = RunOpts { runOptDebug :: Bool
                           , runOptSeparately :: Bool
                           , runOptShowPretty :: Bool}
 
--- read some input from the user
+-- | read some input from the user
 -- parse the input and return the corresponding command
 getCommand :: IO Command
 getCommand = do
     input <- readline "pugs> " 
     doCommand input
 
+doCommand :: Maybe String -> IO Command
 doCommand Nothing = return CmdQuit
 doCommand (Just line)
     | all isSpace line  = getCommand
@@ -46,13 +48,13 @@ doCommand (Just line)
         return $ parseCommandLine line
 
 parseCommandLine :: String -> Command 
-parseCommandLine ('?':str)      = CmdRun (RunOpts True True  True) str
-parseCommandLine ('!':str)      = CmdRun (RunOpts True False True) str
-parseCommandLine ('.':str)      = CmdParse str
+parseCommandLine (':':'e':str)  = CmdRun (RunOpts False True False) str
+parseCommandLine (':':'E':str)  = CmdRun (RunOpts True True False) str
+parseCommandLine (':':'d':str)  = CmdParse str
+parseCommandLine (':':'D':str)  = CmdParseRaw str
 parseCommandLine (':':'q':_)    = CmdQuit
 parseCommandLine (':':'h':_)    = CmdHelp
 parseCommandLine (':':'r':_)    = CmdReset
-parseCommandLine (':':'i':str)  = CmdRun (RunOpts False False False) str
 parseCommandLine (':':'l':str)  = CmdLoad $ unwords (words str)
 parseCommandLine str            = CmdRun (RunOpts False False True) str
 
